"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.GlobalLogger = exports.LoggerContext = exports.LogLevel = void 0;
const deep_js_1 = require("./deep.js");
const global_js_1 = require("./global.js");
const log_console_js_1 = require("./log-console.js");
const log_filter_js_1 = require("./log-filter.js");
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["debug"] = 0] = "debug";
    LogLevel[LogLevel["info"] = 1] = "info";
    LogLevel[LogLevel["warn"] = 2] = "warn";
    LogLevel[LogLevel["error"] = 3] = "error";
    LogLevel[LogLevel["fatal"] = 4] = "fatal";
    LogLevel[LogLevel["off"] = Infinity] = "off";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
function LoggerContext(prefix = "") {
    let logHandlers = [log_console_js_1.LoggerConsoleHandler()];
    let logAssertLevel = LogLevel.warn;
    let logCheckNamespace = (name) => true;
    let logLock = false;
    let logFactory = LoggerBaseFactory;
    function LoggerBaseFactory(name = "") {
        log.extend = function (prefix) {
            return logFactory(name ? `${name}:${prefix}` : prefix);
        };
        const emit = (msg) => {
            if (log.active === true) {
                if (msg.level >= Logger.level && msg.level >= log.level) {
                    if (logCheckNamespace(name)) {
                        for (let handler of logHandlers) {
                            if (handler)
                                handler(msg);
                        }
                    }
                }
            }
        };
        function log(...messages) {
            emit({
                name,
                messages,
                level: LogLevel.debug,
            });
        }
        log.active = true;
        log.level = LogLevel.debug;
        log.debug = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.debug,
            });
        };
        log.info = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.info,
            });
        };
        log.warn = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.warn,
            });
        };
        log.error = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.error,
            });
        };
        // fatal(...args: any[]) {
        //   console.error(...args)
        // },
        log.assert = function (cond, ...messages) {
            if (!cond) {
                if (typeof console !== undefined) {
                    if (console.assert) {
                        // https://developer.mozilla.org/de/docs/Web/API/Console/assert
                        console.assert(cond, ...messages);
                    }
                    else {
                        console.error(`Assert did fail with: ${cond}`, ...messages);
                    }
                }
                emit({
                    name,
                    messages: messages || [`Assert did fail with: ${cond}`],
                    level: logAssertLevel,
                });
                // try {
                //   if (typeof expect !== undefined) {
                //     expect(cond).toBeTruthy()
                //   }
                // } catch (err) {
                //   methods.warn(...args)
                // }
            }
        };
        log.assertEqual = function (value, expected, ...args) {
            let equal = deep_js_1.deepEqual(value, expected);
            if (!equal) {
                log.assert(equal, `Assert did fail. Expected ${expected} got ${value}`, expected, value, ...args);
                // } else {
                //   methods.debug(`Passed equal`)
            }
        };
        log.assertNotEqual = function (value, expected, ...args) {
            let equal = deep_js_1.deepEqual(value, expected);
            if (equal) {
                log.assert(equal, `Assert did fail. Expected ${expected} not to be equal with ${value}`, expected, value, ...args);
                // } else {
                //   methods.debug(`Passed not equal check`)
            }
        };
        return log;
    }
    function Logger(name = "") {
        return logFactory(name);
    }
    Logger.registerHandler = function (handler) {
        logHandlers.push(handler);
    };
    Logger.setFilter = function (namespaces) {
        logCheckNamespace = log_filter_js_1.useNamespaceFilter(namespaces);
    };
    Logger.setLock = (lock = true) => (logLock = lock);
    Logger.setHandlers = function (handlers = []) {
        if (logLock)
            return;
        logHandlers = [...handlers].filter((h) => typeof h === "function");
    };
    Logger.level = LogLevel.debug;
    /** @deprecated */
    Logger.setLogLevel = function (level = 0) {
        if (logLock)
            return;
        Logger.level = level;
    };
    Logger.setFactory = function (factory) {
        if (logLock)
            return;
        logFactory = factory;
    };
    return Logger;
}
exports.LoggerContext = LoggerContext;
// Global logger to guarantee all submodules use the same logger instance
let globalLogger = LoggerContext();
try {
    let _global = global_js_1.getGlobalContext();
    if (_global != null) {
        if ((_global === null || _global === void 0 ? void 0 : _global.logger) == null) {
            _global.logger = globalLogger;
        }
        else {
            globalLogger = _global.logger;
        }
    }
}
catch (e) { }
/** @deprecated Use `Logger` instead, it is global as well */
exports.GlobalLogger = globalLogger;
exports.Logger = globalLogger;
//# sourceMappingURL=log.js.map