"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.SerialQueue = void 0;
// Can learn from here https://github.com/sindresorhus/p-queue
const log_js_1 = require("../common/log.js");
const uuid_js_1 = require("./uuid.js");
const log = log_js_1.Logger("zeed:queue");
/** Guarentee serial execution of tasks. Able to wait, pause, resume and cancel all. */
class SerialQueue {
    constructor(opt = {}) {
        this.queue = [];
        this.isPaused = false;
        this.waitToFinish = [];
        const { name = uuid_js_1.uname("queue"), debug = false } = opt;
        this.name = name;
        this.log = log_js_1.Logger(`zeed:queue:${name}`);
        const logLevel = debug === true ? log_js_1.LogLevel.debug : +debug;
        this.log.active = logLevel >= log_js_1.LogLevel.debug;
        this.log.level = logLevel;
    }
    async performNext() {
        this.log(`performNext, queue.length =`, this.queue.length);
        if (this.currentTask != null) {
            this.log(`performNext => skip while another task is running`);
            return;
        }
        if (this.isPaused) {
            this.log(`performNext => skip while is paused`);
            return;
        }
        while (this.currentTask == null && !this.isPaused) {
            let info = this.queue.shift();
            this.log(`performNext => ${info === null || info === void 0 ? void 0 : info.name}`);
            if (info == null) {
                break;
            }
            const { name, task, resolve } = info;
            this.currentTask = task();
            let result = undefined;
            try {
                this.log.info(`start task ${name}`);
                result = await this.currentTask;
                this.log(`finished task ${name} with result =`, result);
            }
            catch (err) {
                log.warn("Error performing task", err);
            }
            resolve(result);
            this.currentTask = undefined;
        }
        while (this.waitToFinish.length > 0) {
            this.waitToFinish.shift()();
        }
    }
    /** Enqueue task to be executed when all other tasks are done. Except `immediate = true`. */
    async enqueue(task, opt = {}) {
        const { immediate = false, name = uuid_js_1.uname(this.name) } = opt;
        if (immediate) {
            this.log.info(`immediate execution ${name}`);
            return await task();
        }
        this.log(`enqueue ${name}`);
        return new Promise((resolve) => {
            this.queue.push({
                name,
                task,
                resolve,
            });
            this.performNext();
        });
    }
    /** If a task is already performing, execute immediately. Otherwise enqueue as usual. */
    async enqueueReentrant(task, opt = {}) {
        return this.enqueue(task, {
            immediate: this.currentTask != null,
            name: opt.name,
        });
    }
    /** Remove all tasks from queue that are not yet executing. */
    async cancelAll(unblock = true) {
        this.log(`cancelAll`);
        let resolver = this.queue.map((task) => task.resolve);
        this.queue = [];
        resolver.forEach((r) => r(undefined));
        await this.wait();
    }
    /** Pause execution after current task is finished. */
    async pause() {
        this.log(`pause`);
        this.isPaused = true;
        await this.wait();
    }
    /** Resume paused queue. */
    resume() {
        this.log(`resume`);
        this.isPaused = false;
        this.performNext();
    }
    /** Wait for all tasks to finish */
    async wait() {
        this.log(`wait`);
        if (this.currentTask == null &&
            (this.queue.length === 0 || this.isPaused)) {
            return;
        }
        return new Promise((resolve) => {
            this.waitToFinish.push(resolve);
        });
    }
}
exports.SerialQueue = SerialQueue;
//# sourceMappingURL=queue.js.map