// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
import { Logger } from "./log.js";
import { promisify } from "./promise.js";
import { SerialQueue } from "./queue.js";
const log = Logger("zeed:bridge");
export class Encoder {
    encode(data) {
        return JSON.stringify(data);
    }
    decode(data) {
        return JSON.parse(data);
    }
}
export class Bridge {
    constructor(channel, opt = {}) {
        this.encoder = new Encoder();
        this.queue = new SerialQueue();
        this.subscribers = {};
        this.subscribersOnAny = [];
        this.opt = opt;
        this.channel = channel;
        this.channel.on("message", (event) => this._recv(event));
        this.channel.on("messageerror", (event) => {
            log.error("Error in channel", event);
        });
    }
    _post(msg) {
        this.channel.postMessage(this.encoder.encode(msg));
    }
    _recv(ev) {
        let msg = this.encoder.decode(ev.data);
        const event = msg.name;
        const args = msg.info;
        let subscribers = (this.subscribers[event] || []);
        this.subscribersOnAny.forEach((fn) => fn(event, ...args));
        if (subscribers.length > 0) {
            let all = subscribers.map((fn) => {
                try {
                    return promisify(fn(...args));
                }
                catch (err) {
                    log.warn("emit warning:", err);
                }
            });
        }
    }
    emit(event, ...args) {
        try {
            this._post({
                name: event,
                info: args,
            });
        }
        catch (err) {
            log.error("emit exception", err);
        }
        return this;
    }
    async fetch(event, ...args) {
        let result = await this.emit(event, ...args);
        if (Array.isArray(result) && result.length === 1) {
            return result[0];
        }
        return undefined;
    }
    onAny(fn) {
        this.subscribersOnAny.push(fn);
        return this;
    }
    on(event, listener) {
        let subscribers = (this.subscribers[event] || []);
        subscribers.push(listener);
        this.subscribers[event] = subscribers;
        return {
            cleanup: () => {
                this.off(event, listener);
            },
        };
    }
    once(event, listener) {
        const onceListener = async (...args) => {
            this.off(event, onceListener);
            return await promisify(listener(...args));
        };
        this.on(event, onceListener);
        return this;
    }
    off(event, listener) {
        this.subscribers[event] = (this.subscribers[event] || []).filter((f) => listener && f !== listener);
        return this;
    }
    removeAllListeners() {
        this.subscribers = {};
        return this;
    }
}
//# sourceMappingURL=bridge.js.map